Exception: Fusion::FusionError
- Inherits:
-
StandardError
- Object
- StandardError
- Fusion::FusionError
- Defined in:
- lib/fusion.rb
Overview
This gem’s regular error taxonomy. None of these should ever bubble up to the user.
This gem’s regular error taxonomy. None of these should ever bubble up to the user.