Exception: Fusion::Interpreter::Thunk::ReadFailure
- Inherits:
-
StandardError
- Object
- StandardError
- Fusion::Interpreter::Thunk::ReadFailure
- Defined in:
- lib/fusion/interpreter/thunk.rb
Overview
We use a custom Ruby error to transmit read failures between Interpreter.evaluate_file
(which runs in the @compute block) and the Thunk to enforce their connection.
If Interpreter.evaluate_file were to be used outside of a Thunk, the Ruby error would
bubble and trigger an internal_error later on.